import QtQuick 1.1
import com.nokia.meego 1.1
//import com.nokia.extras 1.1
import QtMobility.feedback 1.1

import "../JS/globals.js" as GLOBALS

import "../Common"
import "../Delegates"
import "../Sheets"

Page {
    id: specialItemListView
    tools: specialItemListViewTools
    orientationLock: PageOrientation.LockPortrait

    property string id
    property string pageName
    property string feedType

    function openFile(file, properties) {
             var component = Qt.createComponent(file)
             if (component.status == Component.Ready)
                 pageStack.push(component, properties);
             else
                 console.log("Error loading component:", component.errorString());
         }

    Component.onCompleted: specialItemsModelSql.refresh(feedType, id)
    Component.onDestruction: GLOBALS.previousContentY = 0

    Connections {
        target: folders
        onMarkedReadFolderSuccess: specialItemsModelSql.refresh(feedType, id)
    }
    Connections {
        target: items
        onUpdatedItemsSuccess: { specialItemsModelSql.refresh(feedType, id); itemsList.contentY = GLOBALS.previousContentY; }
        onRequestedItemsSuccess: { specialItemsModelSql.refresh(feedType, id); itemsList.contentY = GLOBALS.previousContentY; }
        onStarredItemsSuccess: { specialItemsModelSql.refresh(feedType, id); itemsList.contentY = GLOBALS.previousContentY; }
        onMarkedItemsSuccess: { specialItemsModelSql.refresh(feedType, id); itemsList.contentY = GLOBALS.previousContentY; }
        onMarkedAllItemsReadSuccess: { specialItemsModelSql.refresh(feedType, id); itemsList.contentY = GLOBALS.previousContentY; }
    }

// ------------- Header Start ----------------

    Header {
        id: specialItemListViewHeader
        text: specialItemListView.pageName
    }

// ------------- Header End ----------------



// ------------- Start Search Field ------------

    Rectangle {
        id: searchFieldBox
        anchors { top: parent.top; topMargin: 71 }
        width: parent.width; height: 0; z: 1
        color: theme.inverted ? "black" : "#E0E1E2"
        clip: true

        Behavior on height { NumberAnimation { duration: 200 } }

        Rectangle {
            id: srect
            anchors { fill: searchFieldBox; leftMargin: 10; rightMargin: 10; top: searchFieldBox.top }
            color: "transparent"
            opacity: searchFieldBox.height === 0 ? 0 : 1

            Behavior on opacity { NumberAnimation { duration: 400 } }



            TextField {
                id: searchTextField
                width: parent.width
                placeholderText: qsTr("Search...")
                anchors { left: srect.left; top: srect.top; topMargin: 9 }
                platformStyle: TextFieldStyle { paddingRight: clearButton.width }
                inputMethodHints: Qt.ImhNoPredictiveText

                onTextChanged: { searchFieldTimer.restart(); specialItemsModelSql.refresh(feedType, id, text) }

                Image {
                    id: clearButton
                    anchors { right: parent.right; verticalCenter: parent.verticalCenter }
                    source: searchTextField.text === "" ? "image://theme/icon-m-common-search" : "image://theme/icon-m-input-clear"
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            searchTextField.text = ""
                            specialItemsModelSql.refresh(feedType, id)
                        }
                    }
                }
            }
        }

        Timer {
            id: searchFieldTimer; running: false; interval: 7000; repeat: false
            onTriggered: { if (searchTextField.text === "") searchFieldBox.height = 0 }
        }
    }

// ------------- End Search Field ------------


    ListView {
        id: itemsList
        anchors { fill: parent; topMargin: searchFieldBox.height + 71; leftMargin: 0; rightMargin: 16 }
        model: specialItemsModelSql
        delegate: SpecialItemListDelegate {
                 onClicked: { GLOBALS.previousContentY = itemsList.contentY; openFile("SingleItemView.qml", {itemId: itemId}) }
                 onPressAndHold: {
                     contextMenuEffect.play()
                     itemContextMenu.id = itemId
                     itemContextMenu.itemTitle = title
                     itemContextMenu.itemUrl = url
                     itemContextMenu.feedName = feedName
                     itemContextMenu.feedId = feedId
                     itemContextMenu.itemGuidHash = guidHash
                     itemContextMenu.starred = starred
                     itemContextMenu.unread = unread
                     itemContextMenu.open()
                 }
        }

        onContentYChanged: {
            if (contentY < -100) {
                searchFieldBox.height = 71;
                searchFieldTimer.running = true;
            }
        }
    }

    ScrollDecorator {
        flickableItem: itemsList
        anchors { right: parent.right; rightMargin: -itemsList.anchors.rightMargin }
    }

    ThemeEffect { id: contextMenuEffect; effect: "LongPress" }


// ------------- ToolBar Start -------------
    ToolBarLayout {
        id: specialItemListViewTools
        visible: true
        ToolIcon {
            platformIconId: "toolbar-back"
            anchors.left: (parent === undefined) ? undefined : parent.left
            onClicked: { pageStack.pop(); }
        }
        ToolIcon {
            id: updaterIcon
            platformIconId: "toolbar-refresh"
            state: feedType != "starred" ? "NORMAL" : "RUNNING"
            states: [
                State {
                    name: "NORMAL"
                    PropertyChanges { target: updaterIcon; enabled: true; visible: true; }
                },
                State {
                    name: "RUNNING"
                    PropertyChanges { target: updaterIcon; enabled: false; visible: false; }
                }
            ]
            onClicked: {
                GLOBALS.previousContentY = itemsList.contentY;
                specialItemListViewHeader.indicatorState = "RUNNING"
                updaterIcon.state = "RUNNING"
                feedType === "folder" ? items.updateItems("0", "1", specialItemListView.id) : generic.initConnection()
            }
            Connections {
                target: items
                onUpdatedItemsSuccess: updaterIcon.state = "NORMAL"
                onUpdatedItemsError: updaterIcon.state = "NORMAL"
            }
            Connections {
                target: generic
                onInitSuccess: updater.startUpdate();
            }
            Connections {
                target: updater
                onUpdateFinished: updaterIcon.state = "NORMAL"
                onUpdateError: updaterIcon.state = "NORMAL"
            }
        }
        ToolIcon {
            platformIconId: "toolbar-view-menu"
            anchors.right: (parent === undefined) ? undefined : parent.right
            onClicked: { (folderMenu.status === DialogStatus.Closed) ? folderMenu.open() : folderMenu.close(); }
            visible: feedType != "starred"
        }
    }

    Menu {
        id: folderMenu
        visualParent: pageStack
        MenuLayout {
            MenuItem {
                text: feedType === "folder" ? qsTr("Mark folder as read") : qsTr("Mark all as read")
                onClicked: {
                    GLOBALS.previousContentY = itemsList.contentY;
                    specialItemListViewHeader.indicatorState = "RUNNING"
                    feedType === "folder" ? folders.markFolderRead(id) : items.markAllItemsRead()
                }
            }
        }
    }


// ----------------- ToolBar End -------------



// ----------------- Context Menu Start --------------

    ContextMenu {
        id: itemContextMenu
        property string id
        property string itemTitle
        property string itemUrl
        property string feedName
        property bool starred
        property string itemGuidHash
        property string feedId
        property bool unread
        function starParams() {
            var feedId=itemContextMenu.feedId;
            var params={};
            params[feedId]=itemContextMenu.itemGuidHash;
            return params;
        }
        function markParams() {
            var params = new Array();
            params[0]=itemContextMenu.itemId;
            return params;
        }
        MenuLayout {
            MenuItem {
                text: itemContextMenu.starred === false ? qsTr("Add to favourites") : qsTr("Remove from favourites")
                onClicked: {
                    GLOBALS.previousContentY = itemsList.contentY;
                    itemContextMenu.starred === false ?
                                    items.starItems("star", itemContextMenu.starParams() ) :
                                    items.starItems("unstar", itemContextMenu.starParams() )
                    specialItemListViewHeader.indicatorState = "RUNNING"
                }
            }
            MenuItem {
                text: itemContextMenu.unread ? qsTr("Mark as read") : qsTr("Mark as unread")
                onClicked: {
                    GLOBALS.previousContentY = itemsList.contentY;
                    specialItemListViewHeader.unread ?
                                items.markItems("read", specialItemListViewHeader.markParams()) :
                                items.markItems("unread", specialItemListViewHeader.markParams());
                    specialItemListViewHeader.indicatorState = "RUNNING"
                }
            }
            MenuItem {
                text: qsTr("Share")
                onClicked: shareUi.share(itemContextMenu.itemUrl, itemContextMenu.itemTitle, itemContextMenu.feedName)
            }
        }
    }

// ----------------- Context Menu End --------------



}
